@@ -228,52 +228,58 @@ class SubscriptionConsumer:
         default_to = self._app.kafka_settings.get("auto_commit_interval_ms", 2000) / 1000
         assignment = None
 
+        to = default_to
         while True:
-            to = default_to
             try:
                 await asyncio.sleep(to)
 
-                # pulled from auto commit handler in aiokafka that
-                # still commits on error
-
-                subscription = self.consumer._subscription.subscription
-                if subscription is not None and not subscription.active:
-                    # The subscription can change few times, so we can not rely on
-                    # flags or topic lists. For example if user changes
-                    # subscription from X to Y and back to X we still need to
-                    # rejoin group.
-                    self.consumer._coordinator.request_rejoin()
-                    subscription = self.consumer._subscription.subscription
-                if subscription is None:
-                    await asyncio.wait(
-                        [self.consumer._subscription.wait_for_subscription()],
-                        return_when=asyncio.FIRST_COMPLETED,
-                    )
-                    subscription = self.consumer._subscription.subscription
-                assert subscription is not None and subscription.active
-                auto_assigned = self.consumer._subscription.partitions_auto_assigned()
-
                 try:
-                    await self.consumer._coordinator.ensure_coordinator_known()
-                    if auto_assigned and self.consumer._coordinator.need_rejoin(subscription):
-                        new_assignment = await self.consumer._coordinator.ensure_active_group(
-                            subscription, assignment
-                        )
-                        if new_assignment is None or not new_assignment.active:
-                            continue
-                        else:
-                            assignment = new_assignment
-                    else:
-                        assignment = subscription.assignment
-
-                    assert assignment is not None and assignment.active
-
-                    to = await self.commit() or default_to
-                except aiokafka.errors.KafkaError:
-                    logger.warning("Error with autocommit", exc_info=True)
-            except (RuntimeError, asyncio.CancelledError, AssertionError):
-                logger.debug("Exiting auto commit task")
+                    assignment = await asyncio.shield(self._ensure_subscription(assignment))
+                    to = await asyncio.shield(self.commit()) or default_to
+                except asyncio.CancelledError:
+                    # cancelled inside tasks should not cause this task to exit
+                    ...
+            except (RuntimeError, asyncio.CancelledError):
+                logger.info("Exiting auto commit task from runtime or cancellation")
                 return
+            except Exception:
+                logger.exception("Unhandled error in auto commit routine, retrying", exc_info=True)
+
+    async def _ensure_subscription(self, assignment: Any) -> Any:
+        # pulled from auto commit handler in aiokafka that
+        # still commits on error
+
+        subscription = self.consumer._subscription.subscription
+        if subscription is not None and not subscription.active:
+            # The subscription can change few times, so we can not rely on
+            # flags or topic lists. For example if user changes
+            # subscription from X to Y and back to X we still need to
+            # rejoin group.
+            self.consumer._coordinator.request_rejoin()
+            subscription = self.consumer._subscription.subscription
+        if subscription is None:
+            await asyncio.wait(
+                [self.consumer._subscription.wait_for_subscription()],
+                return_when=asyncio.FIRST_COMPLETED,
+            )
+            subscription = self.consumer._subscription.subscription
+        assert subscription is not None and subscription.active
+        auto_assigned = self.consumer._subscription.partitions_auto_assigned()
+
+        await self.consumer._coordinator.ensure_coordinator_known()
+        if auto_assigned and self.consumer._coordinator.need_rejoin(subscription):
+            new_assignment = await self.consumer._coordinator.ensure_active_group(
+                subscription, assignment
+            )
+            if new_assignment is None or not new_assignment.active:
+                return assignment
+            else:
+                assignment = new_assignment
+        else:
+            assignment = subscription.assignment
+
+        assert assignment is not None and assignment.active
+        return assignment
 
     async def commit(self) -> Optional[float]:
         """
@@ -283,7 +289,6 @@ class SubscriptionConsumer:
             # it's possible to commit on exit as well
             # so let's make sure to use lock here
             return await self._commit()
-        return None
 
     async def _commit(self) -> float:
         now = time.monotonic()
@@ -298,8 +303,9 @@ class SubscriptionConsumer:
                 if tp not in assignment:
                     # clean if we got a new assignment
                     del to_commit[tp]
-            await self.consumer.commit(to_commit)
-            logging.debug(f"Committed offsets: {to_commit}")
+            if len(to_commit) > 0:
+                await self.consumer.commit(to_commit)
+                logging.debug(f"Committed offsets: {to_commit}")
         except aiokafka.errors.CommitFailedError:
             # try to commit again but we need to combine
             # what could now be in the commit from when we started
